/* This program computes depreciation tables using 
   straight-line and sum-of-the-years'-digits methods.

   Input:  purchase price, salvage value, and useful
           life of an item
   Output: depreciation tables
 -----------------------------------------------------------*/

#include <iostream>         // <<, >>, cout, cin
#include <iomanip>          // output formatters
using namespace std;

int sum (int n);
void straightLine(double amount, int numYears);
void sumOfYears(double amount, int numYears);

int main()
{
  cout << "This program computes depreciation tables using\n"
       << "straight-line and sum-of-the-years'-digits methods.\n\n";

  double purchasePrice,      // item's purchase price,
         salvageValue,       //    salvage value, 
         amount;             //    amount to depreciate, and
  int usefulLife;            //    useful life in years

  cout << "What is the item's purchase price? ";
  cin >> purchasePrice;
  cout << "   salvage value? ";
  cin >> salvageValue;
  cout << "   useful life? ";
  cin >> usefulLife;
  amount = purchasePrice - salvageValue;
  
  straightLine(amount, usefulLife);
  
  sumOfYears(amount, usefulLife);
}

/* Function to compute the sum of the integers from 1 to n.

  Receive: n, an integer
  Return:  the sum 1 + 2 + ... + n (0 if n < 0)
-------------------------------------------------------*/
int sum(int n)
{ 
  return n*(n + 1) / 2;
}

/* Function to output a straight-line depreciation table.

  Receive: amount to depreciate and number of years
  Output:  depreciation table
-------------------------------------------------------*/
void straightLine(double amount, int numYears)
{
   double depreciation = amount / numYears;

   cout << "\nYear -  Depreciation"
        << "\n--------------------\n";

   cout << fixed << showpoint << right       // set up format for $$
        << setprecision(2);

   for (int year = 1; year <= numYears; year++)
      cout << setw(3) << year
           << setw(13) << depreciation << endl;
}

/* Function to output a sum-of-the-years'-digits 
   depreciation table.

   Receive: amount to depreciate and number of years
   Return:  depreciation table
-------------------------------------------------------*/
void sumOfYears(double amount, int numYears)
{
   cout << "\nYear -  Depreciation"
        << "\n--------------------\n";

   double yearSum = sum(numYears);
   double depreciation;

   cout << fixed << showpoint << right       // set up format for $$
        << setprecision(2);

   for (int year = 1; year <= numYears; year++)
   {
      depreciation = (numYears - year + 1) * amount / yearSum;
      cout << setw(3) << year
           << setw(13) << depreciation << endl;
   }
}
